(*

ZZIP compression routines,
adapted for usage with Delphi by A.Myasnikov

e-mail: alexanderwdark@ya.ru

*)

unit LiteZZip;

interface


{$L zziplite.obj}


function ZZCompressBlock (buffer: PByte; size: longword;
  compression_mode: longword; multimedia_test: longword): integer;
  stdcall; external;

function ZZUncompressBlock (buffer: PByte): integer;
  stdcall; external;


type
  TStatusProc = procedure (i1: integer); stdcall;
  PStatusProc = ^TStatusProc;


procedure zzipcompress (from, tof: PChar; seek: integer; bufsize: integer;
  SP: PStatusProc);
procedure zzipdecompress (from, tof: PChar; seek, fsize: integer;
  bufsize: integer; SP: PStatusProc);

implementation

uses SysUtils;

procedure zzipcompress (from, tof: PChar; seek: integer; bufsize: integer;
  SP: PStatusProc);
  var
    Fin, Fout, r, clen: integer;
    buff, destbuff: pointer;
  begin

    GetMem(buff, bufsize);

    GetMem(destbuff, bufsize * 3);

    fin  := FileOpen(from, fmOpenRead);
    fout := FileCreate(tof);
    FileSeek(fout, seek, 0);

    repeat
      r := FileRead(fin, buff^, bufsize);

      if r > 0 then
        begin
        clen := ZZCompressBlock(buff, r, 1, 0);
        FileWrite(fout, clen, 4);
        FileWrite(fout, buff^, clen);
        if SP <> nil then
          TStatusProc(SP)(r);
        end;

    until r = 0;

    FileClose(fin);
    FileClose(fout);
    FreeMem(buff);
    FreeMem(destbuff);

  end;


procedure zzipdecompress (from, tof: PChar; seek, fsize: integer;
  bufsize: integer; SP: PStatusProc);
  var
    Fin, Fout, r, clen, outlen: integer;
    buff: pointer;
  begin

    GetMem(buff, bufsize * 10);


    fin  := FileOpen(from, fmOpenRead);
    fout := FileCreate(tof);
    FileSeek(fin, seek, 0);


    repeat
      r := FileRead(fin, clen, 4);

      if r > 0 then
        begin

        FileRead(fin, buff^, clen);

        outlen := ZzUnCompressBlock(buff);

        FileWrite(fout, buff^, outlen);
        if SP <> nil then
          TStatusProc(SP)(outlen);

        end;

    until r = 0;

    FileClose(fin);
    FileClose(fout);
    FreeMem(buff);

  end;


procedure _free (p: pointer); cdecl;
  begin
    FreeMem(p);
  end;


function _malloc (size: longword): pointer; cdecl;
  begin
    Result := AllocMem(size);
  end;


function _realloc (P: pointer; size: longword): pointer; cdecl;
  begin
    ReallocMem(P, size);
    Result := P;
  end;

function _memset (P: pointer; val: integer; size: integer): pointer; cdecl;
  begin
    system.FillChar(P^, size, val);
    Result := P;
  end;


function _memcpy (Dest: pointer; Src: pointer; size: integer): pointer; cdecl;
  begin
    Move(Src^, Dest^, size);
    Result := Dest;
  end;


end.
